#ifndef VKAR_PHASE_VKARCORR_H
#define VKAR_PHASE_VKARCORR_H

#include <vector>
#include <iostream>
#include <string>
#include "PhaScr.h"
#include "FrameSet.h"

#ifdef HAVE_CCFITS
#include <CCfits>
#endif

/** Characterisation of a two dimenisonal turbulence with van-Karman spectrum.
*/
template <class TYP> class VKarCorr {
public:
	/** The Fried parameter [m].
	* The value refers to an unspecified wavelength. It is the one
	* of the instrument, not necessarily the one at 500 nm.
	*/
	double rFried ;

	/** The outer scale [m].
	* If zero, a Kolmogorov power spectrum is used.
	*/
	double outscal ;

	/** The Karhunen Loeve modes.
	*/
	std::vector<PhaScr<TYP> > modes ;

	/** Phase screens generated.
	*/
	std::vector<PhaScr<TYP> > screens ;

	/** The scaling exponent in the power spectrum.
	* This refers to the 3D turbulence, which is 2/3 for the Kolmogorov
	* rate, *not* to the 2D projected case where this becomes a 5/3.
	*/
	double kolg_exp ;

	VKarCorr<TYP>(const double rho0, const double ouscale=0., const double kolgamma = 2./3.) ;

	double corr(const double d) ;

	void genmods(const FrameSet & fs, const double pixscal) ;

	void genscr(const int Nscr, const int seed, const FrameSet & fs) ;

	void plot(std::ostream & os, const FrameSet & fs) ;

#ifdef HAVE_CCFITS
	CCfits::FITS * toFITS(const std::string & fname,const FrameSet &fs ) const ;
#endif /* HAVE_CCFITS */

protected:
private:

	/** The constant that appears in the relation between the correlation
	* and the factors that depend on outer scale, Fried radius and projected
	* base line.
	*/
	double Cfact ;

	double cphi ;

	/** The outer scale wavenumber, including the factor 2*pi
	*/
	double outscalwn ;

} ; /* VKarCorr */

#endif /* VKAR_PHASE_VKARCORR_H */
